from typing import Optional

from django.db.models import QuerySet

import reversion
from tqdm import tqdm

from aleksis.apps.chronos.util.change_tracker import TimetableDataChangeTracker
from aleksis.apps.untis.util.mysql.importers.terms import import_terms
from aleksis.apps.untis.util.mysql.util import TQDM_DEFAULTS

from .importers.absences import import_absences
from .importers.common_data import (
    import_absence_reasons,
    import_breaks,
    import_classes,
    import_rooms,
    import_subjects,
    import_supervision_areas,
    import_teachers,
    import_time_periods,
)
from .importers.events import import_events
from .importers.holidays import import_holidays
from .importers.lessons import import_lessons
from .importers.substitutions import import_substitutions


def untis_import_mysql(terms: Optional[QuerySet] = None, version: Optional[int] = None):
    # School terms and validity ranges
    validity_ref = import_terms(terms, version=version)

    for validity_range in tqdm(
        validity_ref.values(), desc="Import data for terms", **TQDM_DEFAULTS
    ):
        with reversion.create_revision(atomic=True):
            reversion.set_comment(f"Untis import for validity range {validity_range}")

            tracker = TimetableDataChangeTracker()

            # Common data for Chronos
            subjects_ref = import_subjects(validity_range)
            rooms_ref = import_rooms(validity_range)
            absence_reasons_ref = import_absence_reasons(validity_range)

            # Common data for core
            teachers_ref = import_teachers(validity_range)
            classes_ref = import_classes(validity_range, teachers_ref)

            # Time periods
            time_periods_ref = import_time_periods(validity_range)
            breaks_ref = import_breaks(validity_range, time_periods_ref)

            # Holidays
            holidays_ref = import_holidays(validity_range)

            # Supervisions
            supervision_areas_ref = import_supervision_areas(
                validity_range, breaks_ref, teachers_ref
            )

            # Lessons
            import_lessons(
                validity_range,
                time_periods_ref,
                rooms_ref,
                subjects_ref,
                teachers_ref,
                classes_ref,
            )

            # Substitutions
            import_absences(
                validity_range,
                absence_reasons_ref,
                time_periods_ref,
                teachers_ref,
                classes_ref,
                rooms_ref,
            )
            import_substitutions(
                validity_range,
                teachers_ref,
                subjects_ref,
                rooms_ref,
                classes_ref,
                supervision_areas_ref,
                time_periods_ref,
            )

            # Events
            import_events(validity_range, time_periods_ref, teachers_ref, classes_ref, rooms_ref)



from aleksis.core.util.apps import AppConfig


class UntisConfig(AppConfig):
    name = "aleksis.apps.untis"
    verbose_name = "AlekSIS — Untis interface"

    urls = {
        "Repository": "https://edugit.org/AlekSIS/official/AlekSIS-App-Untis/",
    }
    licence = "EUPL-1.2+"
    copyright_info = (
        ([2018, 2019, 2020, 2021], "Jonathan Weth", "dev@jonathanweth.de"),
        ([2018, 2019], "Frank Poetzsch-Heffter", "p-h@katharineum.de"),
        ([2019, 2020, 2021, 2022], "Dominik George", "dominik.george@teckids.org"),
        ([2019, 2020], "Tom Teichler", "tom.teichler@teckids.org"),
        ([2019], "Julian Leucker", "leuckeju@katharineum.de"),
        ([2019], "mirabilos", "thorsten.glaser@teckids.org"),
    )



import logging

from django.db.models import Q
from django.utils.translation import gettext as _

import reversion
from reversion import set_comment
from tqdm import tqdm

from aleksis.apps.chronos import models as chronos_models
from aleksis.apps.chronos.models import ValidityRange
from aleksis.core import models as core_models
from aleksis.core.util.core_helpers import get_site_preferences

from .... import models as mysql_models
from ....data_checks import CourseGroupNotFoundAndCreated, CourseGroupNotFoundAndNotCreated
from ..util import (
    TQDM_DEFAULTS,
    compare_m2m,
    connect_untis_fields,
    run_default_filter,
    untis_split_third,
)

logger = logging.getLogger(__name__)


def import_lessons(
    validity_range: ValidityRange,
    time_periods_ref,
    rooms_ref,
    subjects_ref,
    teachers_ref,
    classes_ref,
):
    """Import lessons."""
    # Lessons
    lessons = run_default_filter(validity_range, mysql_models.Lesson.objects)

    existing_lessons = []
    for lesson in tqdm(lessons, desc="Import lessons", **TQDM_DEFAULTS):
        lesson_id = lesson.lesson_id

        logger.info(_("Import lesson {}").format(lesson_id))

        if not lesson.lesson_tt:
            logger.warning(_("  Skip because missing times").format(lesson_id))
            continue

        existing_lessons.append(lesson_id)

        # Split data (,)
        raw_lesson_data = connect_untis_fields(lesson, "lessonelement", 10)
        raw_time_data = lesson.lesson_tt.split(",")

        raw_time_data_2 = []
        for el in raw_time_data:
            # Split data (~)
            raw_time_data_2.append(el.split("~"))

        # Get time periods and rooms
        time_periods = []
        rooms_per_periods = []
        for el in raw_time_data_2:
            weekday = int(el[1]) - 1
            hour = int(el[2])
            room_ids = list(filter(lambda r: r != "", el[3].split(";")))

            # Get rooms
            rooms = []
            for room_id in room_ids:
                if room_id and room_id != "0":
                    room_id = int(room_id)
                    r = rooms_ref[room_id]
                else:
                    r = None
                rooms.append(r)

            # Get time period
            time_period = time_periods_ref[weekday][hour]
            time_periods.append(time_period)
            rooms_per_periods.append(rooms)

        # Split data more (~)
        raw_lesson_data_2 = []
        for el in raw_lesson_data:
            raw_lesson_data_2.append(el.split("~"))

        use_room_idx = 0
        current_teacher_id = None
        # All part lessons (courses)
        for i, el in enumerate(raw_lesson_data_2):
            logger.info("  Lesson part {}".format(i))

            # Get plain ids
            teacher_id = int(el[0])
            subject_id = int(el[2])
            class_ids = untis_split_third(el[17], conv=int)

            # Get teacher
            if teacher_id != 0:
                teacher = teachers_ref[teacher_id]
            else:
                teacher = None

            teachers = [teacher] if teacher else []

            # Get subject
            if subject_id != 0:
                subject = subjects_ref[subject_id]
            else:
                logger.warning(_("    Skip because missing subject"))
                continue

            if current_teacher_id and current_teacher_id != teacher_id:
                use_room_idx += 1

            # Get classes
            course_classes = []
            for class_id in class_ids:
                c = classes_ref[class_id]
                course_classes.append(c)

            course_group_not_found_and_not_created = False
            if get_site_preferences()["untis_mysql__use_course_groups"]:
                # Negative import_ref denotes a course group
                group_import_ref = -int("{}{}".format(lesson_id, i))

                # Search by parent groups, teachers/owners and subject
                qs = core_models.Group.objects.filter(
                    parent_groups__in=[c.id for c in course_classes],
                    subject_id=subject.id,
                    owners__in=[t.id for t in teachers],
                ).filter(Q(school_term__isnull=True) | Q(school_term=validity_range.school_term))

                # Check if found groups match
                course_group = None
                for found_group in qs:
                    if compare_m2m(course_classes, found_group.parent_groups.all()) and compare_m2m(
                        teachers, found_group.owners.all()
                    ):
                        course_group = found_group
                        logger.info(
                            "    Course group found by searching by parent groups, "
                            "teachers (owners) and subject"
                        )

                if (
                    not course_group
                    and get_site_preferences()["untis_mysql__course_groups_fuzzy_matching"]
                ):
                    if qs.count() != 1:
                        logger.warning(
                            "    Course group not found by searching by parent groups, "
                            "teachers (owners) and subject (fuzzy matching mode)"
                        )
                    else:
                        for found_group in qs:
                            if compare_m2m(teachers, found_group.owners.all()):
                                if course_group:
                                    logger.warning(
                                        "    More than one course group found "
                                        "by searching by parent groups, "
                                        "teachers (owners) and subject (fuzzy matching mode)"
                                    )
                                    course_group = None
                                else:
                                    course_group = found_group
                                    logger.info(
                                        "    Course group found by searching by parent groups, "
                                        "teachers (owners) and subject (fuzzy matching mode)"
                                    )

                changed = False
                register_data_check = get_site_preferences()[
                    "untis_mysql__data_check_for_not_found_course_groups"
                ]
                if not course_group and get_site_preferences()["untis_mysql__create_course_groups"]:

                    # No matching group found

                    # Build names and refs for course groups
                    group_short_name = "{}-{}".format(
                        "".join([c.short_name for c in course_classes]),
                        subject.short_name,
                    )
                    group_name = "{}: {}".format(
                        ", ".join([c.short_name for c in course_classes]),
                        subject.short_name,
                    )

                    # Get or create course group
                    course_group, created = core_models.Group.objects.get_or_create(
                        short_name=group_short_name, defaults={"name": group_name}
                    )

                    # Log
                    if created:
                        logger.info("    Course group created")

                    # Update parent groups
                    course_group.parent_groups.set(course_classes)
                    logger.info("    Parent groups set")

                    # Update name
                    if course_group.name != group_name:
                        course_group.name = group_name
                        logger.info("    Name of course group updated")

                        changed = True

                    if created and register_data_check:
                        # No match, new course group created, so create a data problem if activated
                        CourseGroupNotFoundAndCreated.register_result(course_group)
                elif register_data_check:
                    # No match, create deactivated, so create a data problem later if activated
                    course_group_not_found_and_not_created = True

                if course_group:
                    # Update owners
                    course_group.owners.set(teachers)

                    # Update import ref
                    if course_group.import_ref_untis != group_import_ref:
                        course_group.import_ref_untis = group_import_ref
                        logger.info("    Import reference of course group updated")
                        changed = True

                    if course_group.subject != subject:
                        course_group.subject = subject
                        logger.info("    Subject reference of course group updated")
                        changed = True

                    if course_group.school_term != validity_range.school_term:
                        course_group.school_term = validity_range.school_term
                        logger.info("    School term reference of course group updated")
                        changed = True

                    if changed:
                        course_group.save()

                    groups = [course_group]

                else:
                    groups = []
            else:
                groups = course_classes

            # Get old lesson
            old_lesson_qs = chronos_models.Lesson.objects.filter(
                lesson_id_untis=lesson_id, element_id_untis=i, validity=validity_range
            )

            if old_lesson_qs.exists():
                # Update existing lesson
                logger.info("    Existing lesson found")

                old_lesson = old_lesson_qs[0]

                if old_lesson.subject != subject:
                    old_lesson.subject = subject
                    old_lesson.save()
                    logger.info("    Subject updated")
                lesson = old_lesson
            else:
                # Create new lesson

                lesson = chronos_models.Lesson.objects.create(
                    subject=subject,
                    lesson_id_untis=lesson_id,
                    element_id_untis=i,
                    validity=validity_range,
                )
                logger.info("    New lesson created")

            if course_group_not_found_and_not_created:
                # Register data problem here because before the lesson object is unknown
                CourseGroupNotFoundAndNotCreated.register_result(lesson)

            # Sync groups
            if groups:
                lesson.groups.set(groups)

            # Sync teachers
            lesson.teachers.set(teachers)

            # All times for this course
            old_lesson_periods_qs = chronos_models.LessonPeriod.objects.filter(lesson=lesson)

            existing_lesson_period_pks = []

            # Sync time periods
            for j, time_period in enumerate(time_periods):
                logger.info("    Import lesson period {}".format(time_period))

                # Get room if provided
                rooms = rooms_per_periods[j]
                if use_room_idx < len(rooms):
                    room = rooms[use_room_idx]
                else:
                    room = None

                # Check if an old lesson period is provided
                old_lesson_period_qs = old_lesson_periods_qs.filter(period=time_period)
                if old_lesson_period_qs.exists():
                    # Update old lesson period

                    lesson_period = old_lesson_period_qs[0]
                    if lesson_period.room != room or lesson_period.element_id_untis != j:
                        lesson_period.element_id_untis = j
                        lesson_period.room = room
                        lesson_period.save()
                        logger.info("      Untis reference and room updated")
                else:
                    # Create new lesson period

                    lesson_period = chronos_models.LessonPeriod.objects.create(
                        lesson=lesson, period=time_period, room=room, element_id_untis=j
                    )
                    logger.info("      New lesson period added")

                existing_lesson_period_pks.append(lesson_period.pk)

            # delete all no-longer existing lesson periods
            old_lesson_periods_qs.exclude(pk__in=existing_lesson_period_pks).delete()
            logger.info("    Old lesson periods deleted")
            current_teacher_id = teacher_id

    for lesson in chronos_models.Lesson.objects.filter(validity=validity_range):
        if lesson.lesson_id_untis and lesson.lesson_id_untis not in existing_lessons:
            logger.info("Lesson {} deleted".format(lesson.id))
            with reversion.create_revision():
                set_comment(_("Deleted by Untis import"))
                lesson.save()
            lesson.delete()



from datetime import date, timedelta

from django.core.management.base import BaseCommand
from django.utils import timezone

from calendarweek import CalendarWeek
from tqdm import tqdm

from aleksis.apps.chronos.models import (
    Absence,
    Event,
    Exam,
    ExtraLesson,
    Holiday,
    LessonSubstitution,
    ValidityRange,
)
from aleksis.core.models import SchoolTerm


class Command(BaseCommand):
    help = "Move all dates to current school year"  # noqa

    def add_arguments(self, parser):
        parser.add_argument("--dry", action="store_true")

    def translate_date_start_end(self, name: str, qs):
        self.stdout.write(name.upper())
        for instance in tqdm(qs):
            date_start = instance.date_start
            date_end = instance.date_end
            date_start_new = date_start + self.time_delta
            date_end_new = date_end + self.time_delta
            self.stdout.write(f"{date_start} → {date_start_new}; {date_end} → {date_end_new}")
            instance.date_start = date_start_new
            instance.date_end = date_end_new
            if not self.dry:
                instance.save()

    def translate_date(self, name: str, qs):
        self.stdout.write(name.upper())
        for instance in tqdm(qs):
            date_old = instance.date
            date_new = date_old + self.time_delta
            self.stdout.write(f"{date_old} → {date_new}")
            instance.date = date_new
            if not self.dry:
                instance.save()

    def translate_week_year(self, name: str, qs):
        self.stdout.write(name.upper())
        for instance in tqdm(qs):
            date_old = instance.date
            date_new = instance.date + self.time_delta
            week_new = CalendarWeek.from_date(date_new)
            self.stdout.write(f"{date_old}, {instance.week} → {date_new}, {week_new.week}")
            instance.year = date_new.year
            instance.week = week_new.week
            if not self.dry:
                instance.save()

    def handle(self, *args, **options):
        self.dry = options["dry"]

        school_terms = SchoolTerm.objects.order_by("-date_end")

        if not school_terms.exists():
            raise RuntimeError("No school term available.")

        school_term = school_terms.first()
        self.stdout.write(f"Used school term: {school_term}")

        date_start = school_term.date_start
        date_end = school_term.date_end
        current_date = timezone.now().date()
        current_year = current_date.year
        if date_start.month <= current_date.month and date_start.day < current_date.day:
            current_year -= 1
        date_end = date(year=current_year, day=date_start.day, month=date_start.month)
        self.stdout.write(f"{current_year}, {date_end}")
        days = (date_end - date_start).days
        self.weeks = round(days / 7)
        self.time_delta = timedelta(days=self.weeks * 7)
        self.stdout.write(f"{days}, {self.weeks}")

        self.translate_date_start_end("SCHOOL TERM", [school_term])

        self.translate_date_start_end(
            "VALIDITY RANGES", ValidityRange.objects.filter(school_term=school_term)
        )

        if not self.weeks:
            self.stdout.write(self.style.ERROR("There are no data that have to be moved."))
            return

        self.translate_week_year(
            "SUBSTITUTIONS",
            LessonSubstitution.objects.filter(
                lesson_period__lesson__validity__school_term=school_term
            ),
        )
        self.translate_week_year(
            "EXTRA LESSONS", ExtraLesson.objects.filter(school_term=school_term)
        )
        self.translate_date_start_end("ABSENCES", Absence.objects.filter(school_term=school_term))
        self.translate_date("EXAMS", Exam.objects.filter(school_term=school_term))
        self.translate_date_start_end(
            "HOLIDAYS", Holiday.objects.within_dates(date_start, date_end)
        )
        self.translate_date_start_end("EVENTS", Event.objects.filter(school_term=school_term))



import logging
from enum import Enum

from tqdm import tqdm

from aleksis.apps.chronos import models as chronos_models
from aleksis.apps.chronos.models import ValidityRange

from .... import models as mysql_models
from ..util import (
    TQDM_DEFAULTS,
    date_to_untis_date,
    get_first_period,
    get_last_period,
    move_weekday_to_range,
    run_default_filter,
    untis_date_to_date,
)

logger = logging.getLogger(__name__)
unknown_reason, _ = chronos_models.AbsenceReason.objects.get_or_create(short_name="?")


class AbsenceType(Enum):
    GROUP = 100
    TEACHER = 101
    ROOM = 102


def import_absences(
    validity_range: ValidityRange,
    absence_reasons_ref,
    time_periods_ref,
    teachers_ref,
    classes_ref,
    rooms_ref,
):
    ref = {}

    untis_term_start = date_to_untis_date(validity_range.date_start)
    untis_term_end = date_to_untis_date(validity_range.date_end)

    # Get absences
    absences = (
        run_default_filter(validity_range, mysql_models.Absence.objects, filter_term=False)
        .filter(datefrom__lte=untis_term_end, dateto__gte=untis_term_start)
        .order_by("absence_id")
    )

    existing_absences = []
    for absence in tqdm(absences, desc="Import absences", **TQDM_DEFAULTS):
        import_ref = absence.absence_id

        logger.info("Import absence {}".format(import_ref))

        if absence.absence_reason_id == 0:
            reason = unknown_reason
        else:
            reason = absence_reasons_ref[absence.absence_reason_id]

        # Build values
        type_ = absence.typea
        date_from = untis_date_to_date(absence.datefrom)
        date_to = untis_date_to_date(absence.dateto)
        period_from = absence.lessonfrom
        period_to = absence.lessonto
        weekday_from = date_from.weekday()
        weekday_to = date_to.weekday()

        # Check min/max weekdays
        weekday_from = move_weekday_to_range(time_periods_ref, weekday_from)
        weekday_to = move_weekday_to_range(time_periods_ref, weekday_to)

        # Check min/max periods
        first_period = get_first_period(time_periods_ref, weekday_from)
        last_period = get_last_period(time_periods_ref, weekday_from)

        if period_from == 0:
            period_from = first_period
        if period_to == 0:
            period_to = last_period

        time_period_from = time_periods_ref[weekday_from][period_from]
        time_period_to = time_periods_ref[weekday_to][period_to]
        comment = absence.text

        group = None
        teacher = None
        room = None

        if type_ == AbsenceType.GROUP.value:
            group = classes_ref[absence.ida]
        elif type_ == AbsenceType.TEACHER.value:
            teacher = teachers_ref[absence.ida]
        elif type_ == AbsenceType.ROOM.value:
            room = rooms_ref[absence.ida]

        new_absence, created = chronos_models.Absence.objects.get_or_create(
            import_ref_untis=import_ref,
            defaults={
                "reason": reason,
                "group": group,
                "teacher": teacher,
                "room": room,
                "date_start": date_from,
                "date_end": date_to,
                "period_from": time_period_from,
                "period_to": time_period_to,
                "comment": absence.text,
                "school_term": validity_range.school_term,
            },
        )

        if created:
            logger.info("  New absence created")

        if (
            new_absence.reason != reason
            or new_absence.group != group
            or new_absence.teacher != teacher
            or new_absence.room != room
            or new_absence.date_start != date_from
            or new_absence.date_end != date_to
            or new_absence.period_from != time_period_from
            or new_absence.period_to != time_period_to
            or new_absence.comment != comment
            or new_absence.school_term != validity_range.school_term
        ):
            new_absence.reason = reason
            new_absence.group = group
            new_absence.teacher = teacher
            new_absence.room = room
            new_absence.date_start = date_from
            new_absence.date_end = date_to
            new_absence.period_from = time_period_from
            new_absence.period_to = time_period_to
            new_absence.comment = comment
            new_absence.school_term = validity_range.school_term
            new_absence.save()
            logger.info("  Absence updated")

        existing_absences.append(import_ref)
        ref[import_ref] = new_absence

    # Delete all no longer existing absences
    for a in chronos_models.Absence.objects.filter(
        date_start__lte=validity_range.date_end, date_end__gte=validity_range.date_start
    ):
        if a.import_ref_untis and a.import_ref_untis not in existing_absences:
            logger.info("Absence {} deleted".format(a.id))
            a.delete()

    return ref



import logging
from datetime import date
from typing import Dict, Optional

from django.db.models import Max, OuterRef, Q, QuerySet, Subquery
from django.utils import timezone

from tqdm import tqdm

from aleksis.apps.chronos import models as chronos_models
from aleksis.apps.untis.util.mysql.util import (
    TQDM_DEFAULTS,
    date_to_untis_date,
    run_using,
    untis_date_to_date,
)
from aleksis.core import models as core_models

from .... import models as mysql_models


def get_terms() -> QuerySet:
    """Get all terms."""
    return run_using(mysql_models.Terms.objects).order_by("datefrom")


def get_terms_for_date_query(for_date: Optional[date] = None) -> QuerySet:
    """Get term query object with term valid for the provided date."""
    if not for_date:
        for_date = timezone.now().date()

    return Q(datefrom__lte=date_to_untis_date(for_date), dateto__gte=date_to_untis_date(for_date))


def get_terms_for_date(for_date: Optional[date] = None) -> QuerySet:
    """Get term queryset with term valid for the provided date."""
    qs = get_terms().filter(get_terms_for_date_query(for_date))

    return qs


def get_future_terms_for_date_query(for_date: Optional[date] = None) -> QuerySet:
    """Get term query object with all future terms."""
    if not for_date:
        for_date = timezone.now().date()

    return Q(datefrom__gt=date_to_untis_date(for_date))


def get_future_terms_for_date(for_date: Optional[date] = None) -> QuerySet:
    """Get all future terms (after the current term)."""
    qs = get_terms().filter(get_future_terms_for_date_query(for_date))

    return qs


logger = logging.getLogger(__name__)


def import_terms(
    qs: Optional[QuerySet] = None,
    version: Optional[int] = None,
) -> Dict[int, chronos_models.ValidityRange]:
    """Import terms and school years as validity ranges and school terms."""
    ranges_ref = {}

    if not isinstance(qs, QuerySet):
        qs = run_using(mysql_models.Terms.objects)

    if version is None:
        # Select newest version per term / validity range
        sub_qs = (
            run_using(mysql_models.Terms.objects)
            .filter(
                school_id=OuterRef("school_id"),
                schoolyear_id=OuterRef("schoolyear_id"),
                term_id=OuterRef("term_id"),
            )
            .values("school_id", "schoolyear_id", "term_id")
            .annotate(max_version=Max("version_id"))
            .values("max_version")
        )
        qs = qs.filter(version_id=Subquery(sub_qs))
    else:
        # Select passed version
        qs = qs.filter(version_id=version)

    school_terms = {}
    for term in tqdm(qs, desc="Import terms (as validity ranges)", **TQDM_DEFAULTS):
        if not term.name:
            raise RuntimeError(
                "Term ID {}: Cannot import term without short name.".format(term.term_id)
            )
        term_id = term.term_id
        name = term.longname if term.longname else term.name
        date_start = untis_date_to_date(term.datefrom)
        date_end = untis_date_to_date(term.dateto)

        logger.info(f"Import term {term_id} ({date_start}–{date_end})")

        school_year_id = term.schoolyear_id
        try:
            school_term = core_models.SchoolTerm.objects.within_dates(date_start, date_end).get()
            logger.info("    School term found by time.")
        except core_models.SchoolTerm.DoesNotExist:
            if school_year_id in school_terms:
                school_term = school_terms[school_year_id]
                logger.info(f"  School year {school_year_id} already there.")
            else:
                school_year = run_using(mysql_models.Schoolyear.objects).get(
                    schoolyear_id=school_year_id
                )
                school_term_name = (
                    school_year.text if school_year.text else school_year.schoolyearzoned
                )

                logger.info(f"  Import school year {school_year_id} ...")

                try:
                    school_term = core_models.SchoolTerm.objects.get(
                        import_ref_untis=school_year_id
                    )
                    logger.info("    School year found by import reference.")
                except core_models.SchoolTerm.DoesNotExist:
                    school_term = core_models.SchoolTerm(
                        date_start=date_start, date_end=date_end, name=school_term_name
                    )
                    logger.info("    School year created newly.")

            school_term.import_ref_untis = school_year_id

        if school_term.date_end < date_end:
            school_term.date_end = date_end

        if school_term.date_start > date_start:
            school_term.date_start = date_start

        school_term.save()

        try:
            validity_range = chronos_models.ValidityRange.objects.get(
                import_ref_untis=term_id, school_term=school_term
            )
            logger.info("  Validity range found by import reference.")
        except chronos_models.ValidityRange.DoesNotExist:
            try:
                validity_range = chronos_models.ValidityRange.objects.within_dates(
                    date_start, date_end
                ).get()
                logger.info("  Validity range found by time.")
            except chronos_models.ValidityRange.DoesNotExist:
                validity_range = chronos_models.ValidityRange()
                logger.info("  Validity range created newly.")

        validity_range.import_ref_untis = term_id
        validity_range.date_start = date_start
        validity_range.date_end = date_end
        validity_range.name = name
        validity_range.school_term = school_term
        validity_range.school_year_untis = school_year_id
        validity_range.school_id_untis = term.school_id
        validity_range.version_id_untis = term.version_id

        validity_range.save()

        ranges_ref[validity_range] = validity_range

    return ranges_ref



from django.utils.translation import gettext_lazy as _

MENUS = {
    "DATA_MANAGEMENT_MENU": [
        {
            "name": _("Link subjects to groups (for Untis MySQL import)"),
            "url": "untis_groups_subjects",
            "validators": [
                (
                    "aleksis.core.util.predicates.permission_validator",
                    "untis.assign_subjects_to_groups_rule",
                ),
            ],
        },
    ]
}
